@@ -1,8 +1,13 @@
 import cython
 from Cython import __version__
 
-from glob import glob
-import re, os, sys
+import re, os, sys, time
+try:
+    from glob import iglob
+except ImportError:
+    # Py2.4
+    from glob import glob as iglob
+
 try:
     import gzip
     gzip_open = gzip.open
@@ -18,13 +23,33 @@ try:
 except ImportError:
     import md5 as hashlib
 
+try:
+    from io import open as io_open
+except ImportError:
+    from codecs import open as io_open
+
+try:
+    from os.path import relpath as _relpath
+except ImportError:
+    # Py<2.6
+    def _relpath(path, start=os.path.curdir):
+        if not path:
+            raise ValueError("no path specified")
+        start_list = os.path.abspath(start).split(os.path.sep)
+        path_list = os.path.abspath(path).split(os.path.sep)
+        i = len(os.path.commonprefix([start_list, path_list]))
+        rel_list = [os.path.pardir] * (len(start_list)-i) + path_list[i:]
+        if not rel_list:
+            return os.path.curdir
+        return os.path.join(*rel_list)
+
 
 from distutils.extension import Extension
 
 from Cython import Utils
-from Cython.Utils import cached_function, cached_method, path_exists
+from Cython.Utils import cached_function, cached_method, path_exists, find_root_package_dir
 from Cython.Compiler.Main import Context, CompilationOptions, default_options
-    
+
 join_path = cached_function(os.path.join)
 
 if sys.version_info[0] < 3:
@@ -44,19 +69,21 @@ def extended_iglob(pattern):
     if '**/' in pattern:
         seen = set()
         first, rest = pattern.split('**/', 1)
-        if first == '':
-            first = '.'
-        for root in glob(first + "/"):
+        if first:
+            first = iglob(first+'/')
+        else:
+            first = ['']
+        for root in first:
             for path in extended_iglob(join_path(root, rest)):
                 if path not in seen:
                     seen.add(path)
                     yield path
-            for path in extended_iglob(join_path(root, '*', '**', rest)):
+            for path in extended_iglob(join_path(root, '*', '**/' + rest)):
                 if path not in seen:
                     seen.add(path)
                     yield path
     else:
-        for path in glob(pattern):
+        for path in iglob(pattern):
             yield path
 
 @cached_function
@@ -206,7 +233,7 @@ def strip_string_literals(code, prefix='__Pyx_L'):
     in_quote = False
     hash_mark = single_q = double_q = -1
     code_len = len(code)
-    
+
     while True:
         if hash_mark < q:
             hash_mark = code.find('#', q)
@@ -286,12 +313,47 @@ def normalize_existing(base_path, rel_paths):
 
 @cached_function
 def normalize_existing0(base_dir, rel_paths):
-    filtered = []
+    normalized = []
     for rel in rel_paths:
         path = join_path(base_dir, rel)
-        if os.path.exists(path):
-            filtered.append(os.path.normpath(path))
-    return filtered
+        if path_exists(path):
+            normalized.append(os.path.normpath(path))
+        else:
+            normalized.append(rel)
+    return normalized
+
+def resolve_depends(depends, include_dirs):
+    include_dirs = tuple(include_dirs)
+    resolved = []
+    for depend in depends:
+        path = resolve_depend(depend, include_dirs)
+        if path is not None:
+            resolved.append(path)
+    return resolved
+
+@cached_function
+def resolve_depend(depend, include_dirs):
+    if depend[0] == '<' and depend[-1] == '>':
+        return None
+    for dir in include_dirs:
+        path = join_path(dir, depend)
+        if path_exists(path):
+            return os.path.normpath(path)
+    return None
+
+@cached_function
+def package(filename):
+    dir = os.path.dirname(os.path.abspath(str(filename)))
+    if dir != filename and path_exists(join_path(dir, '__init__.py')):
+        return package(dir) + (os.path.basename(dir),)
+    else:
+        return ()
+
+@cached_function
+def fully_qualified_name(filename):
+    module = os.path.splitext(os.path.basename(filename))[0]
+    return '.'.join(package(filename) + (module,))
+
 
 @cached_function
 def parse_dependencies(source_filename):
@@ -351,7 +413,7 @@ class DependencyTree(object):
             elif not self.quiet:
                 print("Unable to locate '%s' referenced from '%s'" % (filename, include))
         return all
-    
+
     @cached_method
     def cimports_and_externs(self, filename):
         # This is really ugly. Nested cimports are resolved with respect to the
@@ -368,29 +430,35 @@ class DependencyTree(object):
     def cimports(self, filename):
         return self.cimports_and_externs(filename)[0]
 
-    @cached_method
     def package(self, filename):
-        dir = os.path.dirname(os.path.abspath(str(filename)))
-        if dir != filename and path_exists(join_path(dir, '__init__.py')):
-            return self.package(dir) + (os.path.basename(dir),)
-        else:
-            return ()
+        return package(filename)
+
+    def fully_qualified_name(self, filename):
+        return fully_qualified_name(filename)
 
     @cached_method
-    def fully_qualifeid_name(self, filename):
-        module = os.path.splitext(os.path.basename(filename))[0]
-        return '.'.join(self.package(filename) + (module,))
-
     def find_pxd(self, module, filename=None):
-        if module[0] == '.':
+        is_relative = module[0] == '.'
+        if is_relative and not filename:
             raise NotImplementedError("New relative imports.")
         if filename is not None:
-            relative = '.'.join(self.package(filename) + tuple(module.split('.')))
+            module_path = module.split('.')
+            if is_relative:
+                module_path.pop(0)  # just explicitly relative
+            package_path = list(self.package(filename))
+            while module_path and not module_path[0]:
+                try:
+                    package_path.pop()
+                except IndexError:
+                    return None   # FIXME: error?
+                module_path.pop(0)
+            relative = '.'.join(package_path + module_path)
             pxd = self.context.find_pxd_file(relative, None)
             if pxd:
                 return pxd
+        if is_relative:
+            return None   # FIXME: error?
         return self.context.find_pxd_file(module, None)
-    find_pxd = cached_method(find_pxd)
 
     @cached_method
     def cimported_files(self, filename):
@@ -446,7 +514,7 @@ class DependencyTree(object):
         externs = self.cimports_and_externs(filename)[1]
         if externs:
             if 'depends' in info.values:
-                info.values['depends'] = list(set(info.values['depends']).union(set(externs)))
+                info.values['depends'] = list(set(info.values['depends']).union(externs))
             else:
                 info.values['depends'] = list(externs)
         return info
@@ -499,7 +567,9 @@ def create_dependency_tree(ctx=None, quiet=False):
     return _dep_tree
 
 # This may be useful for advanced users?
-def create_extension_list(patterns, exclude=[], ctx=None, aliases=None, quiet=False):
+def create_extension_list(patterns, exclude=[], ctx=None, aliases=None, quiet=False, exclude_failures=False):
+    if not isinstance(patterns, (list, tuple)):
+        patterns = [patterns]
     explicit_modules = set([m.name for m in patterns if isinstance(m, Extension)])
     seen = set()
     deps = create_dependency_tree(ctx, quiet=quiet)
@@ -507,9 +577,7 @@ def create_extension_list(patterns, exclude=[], ctx=None, aliases=None, quiet=Fa
     if not isinstance(exclude, list):
         exclude = [exclude]
     for pattern in exclude:
-        to_exclude.update(extended_iglob(pattern))
-    if not isinstance(patterns, list):
-        patterns = [patterns]
+        to_exclude.update(map(os.path.abspath, extended_iglob(pattern)))
     module_list = []
     for pattern in patterns:
         if isinstance(pattern, str):
@@ -531,17 +599,22 @@ def create_extension_list(patterns, exclude=[], ctx=None, aliases=None, quiet=Fa
         else:
             raise TypeError(pattern)
         for file in extended_iglob(filepattern):
-            if file in to_exclude:
+            if os.path.abspath(file) in to_exclude:
                 continue
             pkg = deps.package(file)
             if '*' in name:
-                module_name = deps.fully_qualifeid_name(file)
+                module_name = deps.fully_qualified_name(file)
                 if module_name in explicit_modules:
                     continue
             else:
                 module_name = name
             if module_name not in seen:
-                kwds = deps.distutils_info(file, aliases, base).values
+                try:
+                    kwds = deps.distutils_info(file, aliases, base).values
+                except Exception:
+                    if exclude_failures:
+                        continue
+                    raise
                 if base is not None:
                     for key, value in base.values.items():
                         if key not in kwds:
@@ -556,6 +629,12 @@ def create_extension_list(patterns, exclude=[], ctx=None, aliases=None, quiet=Fa
                         if source not in sources:
                             sources.append(source)
                     del kwds['sources']
+                if 'depends' in kwds:
+                    depends = resolve_depends(kwds['depends'], (kwds.get('include_dirs') or []) + [find_root_package_dir(file)])
+                    if template is not None:
+                        # Always include everything from the template.
+                        depends = list(set(template.depends).union(set(depends)))
+                    kwds['depends'] = depends
                 module_list.append(exn_type(
                         name=module_name,
                         sources=sources,
@@ -565,21 +644,67 @@ def create_extension_list(patterns, exclude=[], ctx=None, aliases=None, quiet=Fa
     return module_list
 
 # This is the user-exposed entry point.
-def cythonize(module_list, exclude=[], nthreads=0, aliases=None, quiet=False, force=False, **options):
+def cythonize(module_list, exclude=[], nthreads=0, aliases=None, quiet=False, force=False,
+              exclude_failures=False, **options):
+    """
+    Compile a set of source modules into C/C++ files and return a list of distutils
+    Extension objects for them.
+
+    As module list, pass either a glob pattern, a list of glob patterns or a list of
+    Extension objects.  The latter allows you to configure the extensions separately
+    through the normal distutils options.
+
+    When using glob patterns, you can exclude certain module names explicitly
+    by passing them into the 'exclude' option.
+
+    For parallel compilation, set the 'nthreads' option to the number of
+    concurrent builds.
+
+    For a broad 'try to compile' mode that ignores compilation failures and
+    simply excludes the failed extensions, pass 'exclude_failures=True'. Note
+    that this only really makes sense for compiling .py files which can also
+    be used without compilation.
+
+    Additional compilation options can be passed as keyword arguments.
+    """
     if 'include_path' not in options:
         options['include_path'] = ['.']
+    if 'common_utility_include_dir' in options:
+        if options.get('cache'):
+            raise NotImplementedError, "common_utility_include_dir does not yet work with caching"
+        if not os.path.exists(options['common_utility_include_dir']):
+            os.makedirs(options['common_utility_include_dir'])
     c_options = CompilationOptions(**options)
     cpp_options = CompilationOptions(**options); cpp_options.cplus = True
     ctx = c_options.create_context()
+    options = c_options
     module_list = create_extension_list(
         module_list,
         exclude=exclude,
         ctx=ctx,
         quiet=quiet,
+        exclude_failures=exclude_failures,
         aliases=aliases)
     deps = create_dependency_tree(ctx, quiet=quiet)
+    build_dir = getattr(options, 'build_dir', None)
+    modules_by_cfile = {}
     to_compile = []
     for m in module_list:
+        if build_dir:
+            root = os.path.realpath(os.path.abspath(find_root_package_dir(m.sources[0])))
+            def copy_to_build_dir(filepath, root=root):
+                filepath_abs = os.path.realpath(os.path.abspath(filepath))
+                if os.path.isabs(filepath):
+                    filepath = filepath_abs
+                if filepath_abs.startswith(root):
+                    mod_dir = os.path.join(build_dir,
+                            os.path.dirname(_relpath(filepath, root)))
+                    if not os.path.isdir(mod_dir):
+                        os.makedirs(mod_dir)
+                    shutil.copy(filepath, mod_dir)
+            for dep in m.depends:
+                copy_to_build_dir(dep)
+
         new_sources = []
         for source in m.sources:
             base, ext = os.path.splitext(source)
@@ -590,11 +715,19 @@ def cythonize(module_list, exclude=[], nthreads=0, aliases=None, quiet=False, fo
                 else:
                     c_file = base + '.c'
                     options = c_options
+
+                # setup for out of place build directory if enabled
+                if build_dir:
+                    c_file = os.path.join(build_dir, c_file)
+                    dir = os.path.dirname(c_file)
+                    if not os.path.isdir(dir):
+                        os.makedirs(dir)
+
                 if os.path.exists(c_file):
                     c_timestamp = os.path.getmtime(c_file)
                 else:
                     c_timestamp = -1
-                    
+
                 # Priority goes first to modified files, second to direct
                 # dependents, and finally to indirect dependents.
                 if c_timestamp < deps.timestamp(source):
@@ -614,33 +747,98 @@ def cythonize(module_list, exclude=[], nthreads=0, aliases=None, quiet=False, fo
                         fingerprint = deps.transitive_fingerprint(source, extra)
                     else:
                         fingerprint = None
-                    to_compile.append((priority, source, c_file, fingerprint, quiet, options))
+                    to_compile.append((priority, source, c_file, fingerprint, quiet,
+                                       options, not exclude_failures))
                 new_sources.append(c_file)
+                if c_file not in modules_by_cfile:
+                    modules_by_cfile[c_file] = [m]
+                else:
+                    modules_by_cfile[c_file].append(m)
             else:
                 new_sources.append(source)
+                if build_dir:
+                    copy_to_build_dir(source)
         m.sources = new_sources
     if hasattr(options, 'cache'):
         if not os.path.exists(options.cache):
-            os.mkdir(options.cache)
+            os.makedirs(options.cache)
     to_compile.sort()
     if nthreads:
         # Requires multiprocessing (or Python >= 2.6)
         try:
             import multiprocessing
             pool = multiprocessing.Pool(nthreads)
-            pool.map(cythonize_one_helper, to_compile)
-        except ImportError:
+        except (ImportError, OSError):
             print("multiprocessing required for parallel cythonization")
             nthreads = 0
+        else:
+            pool.map(cythonize_one_helper, to_compile)
     if not nthreads:
         for args in to_compile:
             cythonize_one(*args[1:])
+    if exclude_failures:
+        failed_modules = set()
+        for c_file, modules in modules_by_cfile.iteritems():
+            if not os.path.exists(c_file):
+                failed_modules.update(modules)
+            elif os.path.getsize(c_file) < 200:
+                f = io_open(c_file, 'r', encoding='iso8859-1')
+                try:
+                    if f.read(len('#error ')) == '#error ':
+                        # dead compilation result
+                        failed_modules.update(modules)
+                finally:
+                    f.close()
+        if failed_modules:
+            for module in failed_modules:
+                module_list.remove(module)
+            print("Failed compilations: %s" % ', '.join(sorted([
+                module.name for module in failed_modules])))
     if hasattr(options, 'cache'):
         cleanup_cache(options.cache, getattr(options, 'cache_size', 1024 * 1024 * 100))
+    # cythonize() is often followed by the (non-Python-buffered)
+    # compiler output, flush now to avoid interleaving output.
+    sys.stdout.flush()
     return module_list
 
+
+if os.environ.get('XML_RESULTS'):
+    compile_result_dir = os.environ['XML_RESULTS']
+    def record_results(func):
+        def with_record(*args):
+            t = time.time()
+            success = True
+            try:
+                try:
+                    func(*args)
+                except:
+                    success = False
+            finally:
+                t = time.time() - t
+                module = fully_qualified_name(args[0])
+                name = "cythonize." + module
+                failures = 1 - success
+                if success:
+                    failure_item = ""
+                else:
+                    failure_item = "failure"
+                output = open(os.path.join(compile_result_dir, name + ".xml"), "w")
+                output.write("""
+                    <?xml version="1.0" ?>
+                    <testsuite name="%(name)s" errors="0" failures="%(failures)s" tests="1" time="%(t)s">
+                    <testcase classname="%(name)s" name="cythonize">
+                    %(failure_item)s
+                    </testcase>
+                    </testsuite>
+                """.strip() % locals())
+                output.close()
+        return with_record
+else:
+    record_results = lambda x: x
+
 # TODO: Share context? Issue: pyx processing leaks into pxd module
-def cythonize_one(pyx_file, c_file, fingerprint, quiet, options=None):
+@record_results
+def cythonize_one(pyx_file, c_file, fingerprint, quiet, options=None, raise_on_failure=True):
     from Cython.Compiler.Main import compile, default_options
     from Cython.Compiler.Errors import CompileError, PyrexError
 
@@ -683,9 +881,21 @@ def cythonize_one(pyx_file, c_file, fingerprint, quiet, options=None):
     except (EnvironmentError, PyrexError), e:
         sys.stderr.write('%s\n' % e)
         any_failures = 1
+        # XXX
+        import traceback
+        traceback.print_exc()
+    except Exception:
+        if raise_on_failure:
+            raise
+        import traceback
+        traceback.print_exc()
+        any_failures = 1
     if any_failures:
-        raise CompileError(None, pyx_file)
-    if fingerprint:
+        if raise_on_failure:
+            raise CompileError(None, pyx_file)
+        elif os.path.exists(c_file):
+            os.remove(c_file)
+    elif fingerprint:
         f = open(c_file, 'rb')
         try:
             g = gzip_open(fingerprint_file, 'wb')
@@ -697,7 +907,12 @@ def cythonize_one(pyx_file, c_file, fingerprint, quiet, options=None):
             f.close()
 
 def cythonize_one_helper(m):
-    return cythonize_one(*m[1:])
+    import traceback
+    try:
+        return cythonize_one(*m[1:])
+    except Exception:
+        traceback.print_exc()
+        raise
 
 def cleanup_cache(cache, target_size, ratio=.85):
     try:
