@@ -2,8 +2,8 @@
 
 import os
 import re
-import time
 import codecs
+from xml.sax.saxutils import escape as html_escape
 from StringIO import StringIO
 
 import Version
@@ -11,9 +11,11 @@ from Code import CCodeWriter
 from Cython import Utils
 
 # need one-characters subsitutions (for now) so offsets aren't off
-special_chars = [(u'<', u'\xF0', u'&lt;'),
-                 (u'>', u'\xF1', u'&gt;'),
-                 (u'&', u'\xF2', u'&amp;')]
+special_chars = [
+    (u'&', u'\xF2', u'&amp;'),
+    (u'<', u'\xF0', u'&lt;'),
+    (u'>', u'\xF1', u'&gt;'),
+]
 
 line_pos_comment = re.compile(r'/\*.*?<<<<<<<<<<<<<<.*?\*/\n*', re.DOTALL)
 
@@ -44,7 +46,7 @@ class AnnotationCCodeWriter(CCodeWriter):
         if pos is not None:
             CCodeWriter.mark_pos(self, pos)
         if self.last_pos:
-            pos_code = self.code.setdefault(self.last_pos[0].get_description(),{})
+            pos_code = self.code.setdefault(self.last_pos[0].filename,{})
             code = pos_code.get(self.last_pos[1], "")
             pos_code[self.last_pos[1]] = code + self.annotation_buffer.getvalue()
         self.annotation_buffer = StringIO()
@@ -57,8 +59,7 @@ class AnnotationCCodeWriter(CCodeWriter):
         self.mark_pos(None)
         f = Utils.open_source_file(source_filename)
         lines = f.readlines()
-        for k in range(len(lines)):
-            line = lines[k]
+        for k, line in enumerate(lines):
             for c, cc, html in special_chars:
                 line = line.replace(c, cc)
             lines[k] = line
@@ -75,8 +76,7 @@ class AnnotationCCodeWriter(CCodeWriter):
                     else:
                         all.append((pos, start+end))
 
-        all.sort()
-        all.reverse()
+        all.sort(reverse=True)
         for pos, item in all:
             _, line_no, col = pos
             line_no -= 1
@@ -86,6 +86,8 @@ class AnnotationCCodeWriter(CCodeWriter):
 
         html_filename = os.path.splitext(target_filename)[0] + ".html"
         f = codecs.open(html_filename, "w", encoding="UTF-8")
+        f.write(u'<!DOCTYPE html>\n')
+        f.write(u'<!-- Generated by Cython %s -->\n' % Version.watermark)
         f.write(u'<html>\n')
         f.write(u"""
 <head>
@@ -119,46 +121,41 @@ body { font-family: courier; font-size: 12; }
 <script>
 function toggleDiv(id) {
     theDiv = document.getElementById(id);
-    if (theDiv.style.display == 'none') theDiv.style.display = 'block';
+    if (theDiv.style.display != 'block') theDiv.style.display = 'block';
     else theDiv.style.display = 'none';
 }
 </script>
 </head>
         """)
         f.write(u'<body>\n')
-        f.write(u'<p>Generated by Cython %s on %s\n' % (Version.version, time.asctime()))
+        f.write(u'<p>Generated by Cython %s\n' % Version.watermark)
         c_file = Utils.decode_filename(os.path.basename(target_filename))
         f.write(u'<p>Raw output: <a href="%s">%s</a>\n' % (c_file, c_file))
+
+        zero_calls = dict((name, 0) for name in
+                          'refnanny py_macro_api py_c_api pyx_macro_api pyx_c_api error_goto'.split())
+
+        def annotate(match):
+            group_name = match.lastgroup
+            calls[group_name] += 1
+            return ur"<span class='%s'>%s</span>" % (
+                group_name, match.group(group_name))
+
         k = 0
-
-        py_c_api = re.compile(u'(Py[A-Z][a-z]+_[A-Z][a-z][A-Za-z_]+)\(')
-        py_marco_api = re.compile(u'(Py[A-Z][a-z]+_[A-Z][A-Z_]+)\(')
-        pyx_c_api = re.compile(u'(__Pyx_[A-Z][a-z_][A-Za-z_]+)\(')
-        pyx_macro_api = re.compile(u'(__Pyx_[A-Z][A-Z_]+)\(')
-        error_goto = re.compile(ur'((; *if .*)? \{__pyx_filename = .*goto __pyx_L\w+;\})')
-        refnanny = re.compile(u'(__Pyx_X?(GOT|GIVE)REF|__Pyx_RefNanny[A-Za-z]+)')
-
-        code_source_file = self.code[source_filename]
+        code_source_file = self.code.get(source_filename, {})
         for line in lines:
-
             k += 1
             try:
                 code = code_source_file[k]
             except KeyError:
                 code = ''
+            else:
+                code = html_escape(code)
 
-            code = code.replace('<', '<code><</code>')
-
-            code, py_c_api_calls = py_c_api.subn(ur"<span class='py_c_api'>\1</span>(", code)
-            code, pyx_c_api_calls = pyx_c_api.subn(ur"<span class='pyx_c_api'>\1</span>(", code)
-            code, py_macro_api_calls = py_marco_api.subn(ur"<span class='py_macro_api'>\1</span>(", code)
-            code, pyx_macro_api_calls = pyx_macro_api.subn(ur"<span class='pyx_macro_api'>\1</span>(", code)
-            code, refnanny_calls = refnanny.subn(ur"<span class='refnanny'>\1</span>", code)
-            code, error_goto_calls = error_goto.subn(ur"<span class='error_goto'>\1</span>", code)
-
-            code = code.replace(u"<span class='error_goto'>;", u";<span class='error_goto'>")
-
-            score = 5*py_c_api_calls + 2*pyx_c_api_calls + py_macro_api_calls + pyx_macro_api_calls - refnanny_calls
+            calls = zero_calls.copy()
+            code = _parse_code(annotate, code)
+            score = (5 * calls['py_c_api'] + 2 * calls['pyx_c_api'] +
+                     calls['py_macro_api'] + calls['pyx_macro_api'])
             color = u"FFFF%02x" % int(255/(1+score/10.0))
             f.write(u"<pre class='line' style='background-color: #%s' onclick='toggleDiv(\"line%s\")'>" % (color, k))
 
@@ -174,13 +171,16 @@ function toggleDiv(id) {
         f.close()
 
 
-# TODO: make this cleaner
-def escape(raw_string):
-    raw_string = raw_string.replace(u"\'", ur"&#146;")
-    raw_string = raw_string.replace(u'\"', ur'&quot;')
-    raw_string = raw_string.replace(u'\n', ur'<br>\n')
-    raw_string = raw_string.replace(u'\t', ur'\t')
-    return raw_string
+_parse_code = re.compile(
+    ur'(?P<refnanny>__Pyx_X?(?:GOT|GIVE)REF|__Pyx_RefNanny[A-Za-z]+)|'
+    ur'(?:'
+    ur'(?P<pyx_macro_api>__Pyx_[A-Z][A-Z_]+)|'
+    ur'(?P<pyx_c_api>__Pyx_[A-Z][a-z_][A-Za-z_]+)|'
+    ur'(?P<py_macro_api>Py[A-Z][a-z]+_[A-Z][A-Z_]+)|'
+    ur'(?P<py_c_api>Py[A-Z][a-z]+_[A-Z][a-z][A-Za-z_]+)'
+    ur')(?=\()|'       # look-ahead to exclude subsequent '(' from replacement
+    ur'(?P<error_goto>(?:(?<=;) *if .* +)?\{__pyx_filename = .*goto __pyx_L\w+;\})'
+).sub
 
 
 class AnnotationItem(object):
