@@ -172,8 +172,10 @@ def unpack_source_tree(tree_file, dir=None):
     header = []
     cur_file = None
     f = open(tree_file)
-    lines = f.readlines()
-    f.close()
+    try:
+        lines = f.readlines()
+    finally:
+        f.close()
     f = None
     for line in lines:
         if line[:5] == '#####':
@@ -186,8 +188,9 @@ def unpack_source_tree(tree_file, dir=None):
             cur_file = open(path, 'w')
         elif cur_file is not None:
             cur_file.write(line)
-        else:
-            header.append(line)
+        elif line.strip() and not line.lstrip().startswith('#'):
+            if line.strip() not in ('"""', "'''"):
+                header.append(line)
     if cur_file is not None:
         cur_file.close()
     return dir, ''.join(header)
