@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 
 import os
 import sys
@@ -58,6 +58,13 @@ except ImportError:
             self._dict[key] = value
         def __repr__(self):
             return repr(self._dict)
+        def __nonzero__(self):
+            return bool(self._dict)
+
+try:
+    basestring
+except NameError:
+    basestring = str
 
 WITH_CYTHON = True
 CY3_DIR = None
@@ -87,10 +94,38 @@ EXT_DEP_MODULES = {
     'tag:array' : 'array',
 }
 
+def patch_inspect_isfunction():
+    import inspect
+    orig_isfunction = inspect.isfunction
+    def isfunction(obj):
+        return orig_isfunction(obj) or type(obj).__name__ == 'cython_function_or_method'
+    isfunction._orig_isfunction = orig_isfunction
+    inspect.isfunction = isfunction
+
+def unpatch_inspect_isfunction():
+    import inspect
+    try:
+        orig_isfunction = inspect.isfunction._orig_isfunction
+    except AttributeError:
+        pass
+    else:
+        inspect.isfunction = orig_isfunction
+
+def update_linetrace_extension(ext):
+    ext.define_macros.append(('CYTHON_TRACE', 1))
+    return ext
+
 def update_numpy_extension(ext):
     import numpy
+    from numpy.distutils.misc_util import get_info
+
     ext.include_dirs.append(numpy.get_include())
 
+    # We need the npymath library for numpy.math.
+    # This is typically a static-only library.
+    for attr, value in get_info('npymath').items():
+        getattr(ext, attr).extend(value)
+
 def update_openmp_extension(ext):
     ext.openmp = True
     language = ext.language
@@ -169,6 +204,7 @@ EXCLUDE_EXT = object()
 EXT_EXTRAS = {
     'tag:numpy' : update_numpy_extension,
     'tag:openmp': update_openmp_extension,
+    'tag:trace':  update_linetrace_extension,
 }
 
 # TODO: use tags
@@ -183,9 +219,15 @@ VER_DEP_MODULES = {
                                           'run.all',
                                           'run.yield_from_pep380',  # GeneratorExit
                                           'run.generator_frame_cycle', # yield in try-finally
+                                          'run.generator_expressions_in_class',
+                                          'run.absolute_import',
                                           'run.relativeimport_T542',
                                           'run.relativeimport_star_T542',
                                           'run.initial_file_path',  # relative import
+                                          'run.pynumber_subtype_conversion',  # bug in Py2.4
+                                          'build.cythonize_script',  # python2.4 -m a.b.c
+                                          'build.cythonize_script_excludes',  # python2.4 -m a.b.c
+                                          'build.cythonize_script_package',  # python2.4 -m a.b.c
                                           ]),
     (2,6) : (operator.lt, lambda x: x in ['run.print_function',
                                           'run.language_level', # print function
@@ -195,7 +237,12 @@ VER_DEP_MODULES = {
                                           'run.pure_py', # decorators, with statement
                                           'run.purecdef',
                                           'run.struct_conversion',
+                                          'run.bytearray_coercion',
+                                          'run.bytearraymethods',
+                                          'run.bytearray_ascii_auto_encoding',
+                                          'run.bytearray_default_auto_encoding',
                                           # memory views require buffer protocol
+                                          'memoryview.relaxed_strides',
                                           'memoryview.cythonarray',
                                           'memoryview.memslice',
                                           'memoryview.numpy_memoryview',
@@ -204,6 +251,7 @@ VER_DEP_MODULES = {
                                           ]),
     (2,7) : (operator.lt, lambda x: x in ['run.withstat_py', # multi context with statement
                                           'run.yield_inside_lambda',
+                                          'run.test_dictviews',
                                           ]),
     # The next line should start (3,); but this is a dictionary, so
     # we can only have one (3,) key.  Since 2.7 is supposed to be the
@@ -216,6 +264,8 @@ VER_DEP_MODULES = {
                                         'compile.extsetslice',
                                         'compile.extdelslice',
                                         'run.special_methods_T561_py2']),
+    (3,3) : (operator.lt, lambda x: x in ['build.package_compilation',
+                                          ]),
 }
 
 # files that should not be converted to Python 3 code with 2to3
@@ -234,6 +284,9 @@ TEST_SUPPORT_DIR = 'testsupport'
 
 BACKENDS = ['c', 'cpp']
 
+UTF8_BOM_BYTES = r'\xef\xbb\xbf'.encode('ISO-8859-1').decode('unicode_escape')
+
+
 def memoize(f):
     uncomputed = object()
     f._cache = {}
@@ -244,22 +297,37 @@ def memoize(f):
         return res
     return func
 
+
 @memoize
 def parse_tags(filepath):
     tags = defaultdict(list)
-    f = io_open(filepath, encoding='ISO-8859-1', errors='replace')
+    parse_tag = re.compile(r'#\s*(\w+)\s*:(.*)$').match
+    f = io_open(filepath, encoding='ISO-8859-1', errors='ignore')
     try:
         for line in f:
-            line = line.strip()
+            # ignore BOM-like bytes and whitespace
+            line = line.lstrip(UTF8_BOM_BYTES).strip()
             if not line:
-                continue
+                if tags:
+                    break  # assume all tags are in one block
+                else:
+                    continue
             if line[0] != '#':
                 break
-            ix = line.find(':')
-            if ix != -1:
-                tag = line[1:ix].strip()
-                values = line[ix+1:].split(',')
-                tags[tag].extend([value.strip() for value in values])
+            parsed = parse_tag(line)
+            if parsed:
+                tag, values = parsed.groups()
+                if tag in ('coding', 'encoding'):
+                    continue
+                if tag == 'tags':
+                    tag = 'tag'
+                    print("WARNING: test tags use the 'tag' directive, not 'tags' (%s)" % filepath)
+                if tag not in ('mode', 'tag', 'ticket', 'cython'):
+                    print("WARNING: unknown test directive '%s' found (%s)" % (tag, filepath))
+                values = values.split(',')
+                tags[tag].extend(filter(None, [value.strip() for value in values]))
+            elif tags:
+                break  # assume all tags are in one block
     finally:
         f.close()
     return tags
@@ -267,6 +335,20 @@ def parse_tags(filepath):
 list_unchanging_dir = memoize(lambda x: os.listdir(x))
 
 
+def import_ext(module_name, file_path=None):
+    if file_path:
+        import imp
+        return imp.load_dynamic(module_name, file_path)
+    else:
+        try:
+            from importlib import invalidate_caches
+        except ImportError:
+            pass
+        else:
+            invalidate_caches()
+        return __import__(module_name, globals(), locals(), ['*'])
+
+
 class build_ext(_build_ext):
     def build_extension(self, ext):
         try:
@@ -366,7 +448,10 @@ class TestBuilder(object):
                 continue
             if filename.startswith('.'):
                 continue # certain emacs backup files
-            tags = parse_tags(filepath)
+            if context == 'pyregr':
+                tags = defaultdict(list)
+            else:
+                tags = parse_tags(filepath)
             fqmodule = "%s.%s" % (context, module)
             if not [ 1 for match in self.selectors
                      if match(fqmodule, tags) ]:
@@ -422,9 +507,13 @@ class TestBuilder(object):
                 languages = self.languages[:1]
         else:
             languages = self.languages
+
         if 'cpp' in tags['tag'] and 'c' in languages:
             languages = list(languages)
             languages.remove('c')
+        elif 'no-cpp' in tags['tag'] and 'cpp' in self.languages:
+            languages = list(languages)
+            languages.remove('cpp')
         tests = [ self.build_test(test_class, path, workdir, module,
                                   language, expect_errors, warning_errors)
                   for language in languages ]
@@ -474,9 +563,14 @@ class CythonCompileTestCase(unittest.TestCase):
     def setUp(self):
         from Cython.Compiler import Options
         self._saved_options = [ (name, getattr(Options, name))
-                                for name in ('warning_errors', 'error_on_unknown_names') ]
+                                for name in ('warning_errors',
+                                             'clear_to_none',
+                                             'error_on_unknown_names',
+                                             'error_on_uninitialized') ]
         self._saved_default_directives = Options.directive_defaults.items()
         Options.warning_errors = self.warning_errors
+        if sys.version_info >= (3, 4):
+            Options.directive_defaults['autotestdict'] = False
 
         if not os.path.exists(self.workdir):
             os.makedirs(self.workdir)
@@ -488,6 +582,7 @@ class CythonCompileTestCase(unittest.TestCase):
         for name, value in self._saved_options:
             setattr(Options, name, value)
         Options.directive_defaults = dict(self._saved_default_directives)
+        unpatch_inspect_isfunction()
 
         try:
             sys.path.remove(self.workdir)
@@ -527,8 +622,9 @@ class CythonCompileTestCase(unittest.TestCase):
         self.success = True
 
     def runCompileTest(self):
-        self.compile(self.test_directory, self.module, self.workdir,
-                     self.test_directory, self.expect_errors, self.annotate)
+        return self.compile(
+            self.test_directory, self.module, self.workdir,
+            self.test_directory, self.expect_errors, self.annotate)
 
     def find_module_source_file(self, source_file):
         if not os.path.exists(source_file):
@@ -671,6 +767,24 @@ class CythonCompileTestCase(unittest.TestCase):
         finally:
             os.chdir(cwd)
 
+        try:
+            get_ext_fullpath = build_extension.get_ext_fullpath
+        except AttributeError:
+            def get_ext_fullpath(ext_name, self=build_extension):
+                # copied from distutils.command.build_ext (missing in Py2.[45])
+                fullname = self.get_ext_fullname(ext_name)
+                modpath = fullname.split('.')
+                filename = self.get_ext_filename(modpath[-1])
+                if not self.inplace:
+                    filename = os.path.join(*modpath[:-1]+[filename])
+                    return os.path.join(self.build_lib, filename)
+                package = '.'.join(modpath[0:-1])
+                build_py = self.get_finalized_command('build_py')
+                package_dir = os.path.abspath(build_py.get_package_dir(package))
+                return os.path.join(package_dir, filename)
+
+        return get_ext_fullpath(module)
+
     def compile(self, test_directory, module, workdir, incdir,
                 expect_errors, annotate):
         expected_errors = errors = ()
@@ -705,11 +819,20 @@ class CythonCompileTestCase(unittest.TestCase):
                 print('\n'.join(errors))
                 print('\n')
                 raise
+            return None
+
+        if self.cython_only:
+            so_path = None
         else:
-            if not self.cython_only:
-                self.run_distutils(test_directory, module, workdir, incdir)
+            so_path = self.run_distutils(test_directory, module, workdir, incdir)
+        return so_path
 
 class CythonRunTestCase(CythonCompileTestCase):
+    def setUp(self):
+        CythonCompileTestCase.setUp(self)
+        from Cython.Compiler import Options
+        Options.clear_to_none = False
+
     def shortDescription(self):
         if self.cython_only:
             return CythonCompileTestCase.shortDescription(self)
@@ -724,9 +847,10 @@ class CythonRunTestCase(CythonCompileTestCase):
             self.setUp()
             try:
                 self.success = False
-                self.runCompileTest()
+                ext_so_path = self.runCompileTest()
                 failures, errors = len(result.failures), len(result.errors)
-                self.run_tests(result)
+                if not self.cython_only:
+                    self.run_tests(result, ext_so_path)
                 if failures == len(result.failures) and errors == len(result.errors):
                     # No new errors...
                     self.success = True
@@ -740,24 +864,27 @@ class CythonRunTestCase(CythonCompileTestCase):
         except Exception:
             pass
 
-    def run_tests(self, result):
-        if not self.cython_only:
-            self.run_doctests(self.module, result)
+    def run_tests(self, result, ext_so_path):
+        self.run_doctests(self.module, result, ext_so_path)
 
-    def run_doctests(self, module_name, result):
+    def run_doctests(self, module_or_name, result, ext_so_path):
         def run_test(result):
-            tests = doctest.DocTestSuite(module_name)
+            if isinstance(module_or_name, basestring):
+                module = import_ext(module_or_name, ext_so_path)
+            else:
+                module = module_or_name
+            tests = doctest.DocTestSuite(module)
             tests.run(result)
         run_forked_test(result, run_test, self.shortDescription(), self.fork)
 
-
 def run_forked_test(result, run_func, test_name, fork=True):
     if not fork or sys.version_info[0] >= 3 or not hasattr(os, 'fork'):
         run_func(result)
+        sys.stdout.flush()
+        sys.stderr.flush()
         gc.collect()
         return
 
-    module_name = test_name.split()[-1]
     # fork to make sure we do not keep the tested module loaded
     result_handle, result_file = tempfile.mkstemp()
     os.close(result_handle)
@@ -770,6 +897,8 @@ def run_forked_test(result, run_func, test_name, fork=True):
                 try:
                     partial_result = PartialTestResult(result)
                     run_func(partial_result)
+                    sys.stdout.flush()
+                    sys.stderr.flush()
                     gc.collect()
                 except Exception:
                     if tests is None:
@@ -798,7 +927,7 @@ def run_forked_test(result, run_func, test_name, fork=True):
         if result_code & 255:
             raise Exception("Tests in module '%s' were unexpectedly killed by signal %d"%
                             (module_name, result_code & 255))
-        result_code = result_code >> 8
+        result_code >>= 8
         if result_code in (0,1):
             input = open(result_file, 'rb')
             try:
@@ -903,8 +1032,9 @@ class CythonUnitTestCase(CythonRunTestCase):
     def shortDescription(self):
         return "compiling (%s) tests in %s" % (self.language, self.module)
 
-    def run_tests(self, result):
-        unittest.defaultTestLoader.loadTestsFromName(self.module).run(result)
+    def run_tests(self, result, ext_so_path):
+        module = import_ext(self.module, ext_so_path)
+        unittest.defaultTestLoader.loadTestsFromModule(module).run(result)
 
 
 class CythonPyregrTestCase(CythonRunTestCase):
@@ -912,9 +1042,11 @@ class CythonPyregrTestCase(CythonRunTestCase):
         CythonRunTestCase.setUp(self)
         from Cython.Compiler import Options
         Options.error_on_unknown_names = False
+        Options.error_on_uninitialized = False
         Options.directive_defaults.update(dict(
             binding=True, always_allow_keywords=True,
             set_initial_path="SOURCEFILE"))
+        patch_inspect_isfunction()
 
     def _run_unittest(self, result, *classes):
         """Run tests from unittest.TestCase-derived classes."""
@@ -933,9 +1065,9 @@ class CythonPyregrTestCase(CythonRunTestCase):
         suite.run(result)
 
     def _run_doctest(self, result, module):
-        self.run_doctests(module, result)
+        self.run_doctests(module, result, None)
 
-    def run_tests(self, result):
+    def run_tests(self, result, ext_so_path):
         try:
             from test import support
         except ImportError: # Python2.x
@@ -947,15 +1079,22 @@ class CythonPyregrTestCase(CythonRunTestCase):
             def run_doctest(module, verbosity=None):
                 return self._run_doctest(result, module)
 
+            backup = (support.run_unittest, support.run_doctest)
             support.run_unittest = run_unittest
             support.run_doctest = run_doctest
 
             try:
-                module = __import__(self.module)
-                if hasattr(module, 'test_main'):
-                    module.test_main()
-            except (unittest.SkipTest, support.ResourceDenied):
-                result.addSkip(self, 'ok')
+                try:
+                    sys.stdout.flush() # helps in case of crashes
+                    module = import_ext(self.module, ext_so_path)
+                    sys.stdout.flush() # helps in case of crashes
+                    if hasattr(module, 'test_main'):
+                        module.test_main()
+                        sys.stdout.flush() # helps in case of crashes
+                except (unittest.SkipTest, support.ResourceDenied):
+                    result.addSkip(self, 'ok')
+            finally:
+                support.run_unittest, support.run_doctest = backup
 
         run_forked_test(result, run_test, self.shortDescription(), self.fork)
 
@@ -1103,8 +1242,8 @@ class EndToEndTest(unittest.TestCase):
         old_path = os.environ.get('PYTHONPATH')
         os.environ['PYTHONPATH'] = self.cython_syspath + os.pathsep + (old_path or '')
         try:
-            for command in commands.split('\n'):
-                p = subprocess.Popen(commands,
+            for command in filter(None, commands.splitlines()):
+                p = subprocess.Popen(command,
                                      stderr=subprocess.PIPE,
                                      stdout=subprocess.PIPE,
                                      shell=True)
